@@ -1,5 +1,5 @@
 /*  Audacious-DUMB Plug-in to enable audacious to use DUMB for IT/XM/S3M/MOD files
- *  Copyright (C) 2006-2007 Christian Birchinger
+ *  Copyright (C) 2006-2011 Christian Birchinger
  *  Audacious port based on previous plugins for XMMS and BMP
  *  Copyright (C) 2002-2003  Ben Davis
  *  Incorporates code from the wav plug-in,
@@ -22,61 +22,63 @@
 
 #include "audacious-dumb.h"
 
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include "interface.h"
-#include "support.h"
+#include <audacious/i18n.h>
+#include <audacious/gtk-compat.h>
 
 DuhFile *duh_file = NULL;
-static gboolean disable_amiga_mods = FALSE;
-static const gchar duh_custom_title_format[] = "${file-name}${?title: - ${title}}";
-static gint output_frequency;
-static const gint output_frequencies[]={22050, 44100, 48000, 64000, 96000, 192000};
-static GtkWidget *prefs;
 static GMutex *dumblock;
 
+dumb_config_t dumb_config;
+dumb_config_t dumb_config_gui;
+
 static GMutex *control_mutex;
 static GCond *control_cond;
 static gboolean stop_flag = FALSE;
 
 static const gchar *fmts[] = { "mod", "s3m", "it", "xm", "duh", NULL };
 
-InputPlugin duh_ip = {
-    .description = "DUMB Plugin " VERSION,
-    .init = duh_init,
-    .cleanup = duh_cleanup,
-    .about = duh_about,
-    .configure = duh_configure,
-    .play = play_start,
-    .stop = stop,
-    .pause = duh_pause,
-    .file_info_box = file_info_box,
-    .is_our_file_from_vfs = is_our_file_from_vfs,
-    .vfs_extensions = fmts,
-    .mseek = mseek,
-    .probe_for_tuple = duh_probe_for_tuple,
-    .priority = 0 /* Would be nice to be above Modplug but minus values don't work */ 
-};
+static const gchar * const DUMB_defaults[] = {
+	"disable_amiga_mods", "FALSE",
+	"output_frequency", "44100",
+	"use_custom_title", "TRUE",
+	NULL};
 
-InputPlugin *duh_iplist[] = { &duh_ip, NULL };
-
-SIMPLE_INPUT_PLUGIN(duh_ip, duh_iplist)
+AUD_INPUT_PLUGIN
+(
+	.name = "DUMB Plugin " VERSION,
+	.init = duh_init,
+	.cleanup = duh_cleanup,
+	.about = duh_about,
+	.settings = &preferences,
+	.play = play_start,
+	.stop = stop,
+	.pause = duh_pause,
+	.file_info_box = file_info_box,
+	.is_our_file_from_vfs = is_our_file_from_vfs,
+	.extensions = fmts,
+	.mseek = mseek,
+	.probe_for_tuple = duh_probe_for_tuple,
+	.priority = 0 /* Would be nice to be above Modplug but minus values don't work */ 
+)
 
 static gboolean duh_init(void)
 {
-	mcs_handle_t *db;
-	db = aud_cfg_db_open();
+	aud_config_set_defaults ("DUMB", DUMB_defaults);
 
-	aud_cfg_db_get_bool(db, "DUMB", "disable_amiga_mods", &disable_amiga_mods);
-	if (!disable_amiga_mods) /* force db creation since defined false equals none-existant */
-		aud_cfg_db_set_bool(db, "DUMB", "disable_amiga_mods", FALSE);
+	memset(&dumb_config, 0, sizeof(dumb_config));
 
-	aud_cfg_db_get_int(db, "DUMB", "output_frequency", &output_frequency);	
-	if (output_frequency < 22050 || output_frequency > 192000) {
-		output_frequency = 44100;
-		aud_cfg_db_set_int(db, "DUMB", "output_frequency", output_frequency);
+	dumb_config.disable_amiga_mods = aud_get_bool("DUMB", "disable_amiga_mods");
+
+	dumb_config.output_frequency = aud_get_int("DUMB", "output_frequency");
+
+	if (dumb_config.output_frequency < 22050 || dumb_config.output_frequency > 192000) {
+		dumb_config.output_frequency = 44100;
+		aud_set_int ("DUMB", "output_frequency", dumb_config.output_frequency);
 	}
-		
-	aud_cfg_db_close(db);
+
+	dumb_config.use_custom_title = aud_get_bool("DUMB", "use_custom_title");
 
 	/* dumb_register_stdfiles(); */
 	/*  Seems only needed when opening files with dumb directly * 
@@ -95,7 +97,7 @@ static void duh_cleanup(void)
 {
 	g_mutex_free(dumblock);
 	g_mutex_free(control_mutex);
-	g_cond_free(control_cond);  
+	g_cond_free(control_cond);
 }	
 
 static int is_our_file_from_vfs(const gchar *filename, VFSFile *file)
@@ -106,19 +108,24 @@ static int is_our_file_from_vfs(const gchar *filename, VFSFile *file)
 	g_return_val_if_fail(filename != NULL, FALSE);
 	g_return_val_if_fail(file != NULL, FALSE);
 
-	vfs_fread(magic, 1, 4, file);
+	if (vfs_fread(magic, 1, 4, file) != sizeof magic)
+		return FALSE;
 	if (!memcmp(magic, XM_MAGIC, 4))
                 return TRUE;
 	if (!memcmp(magic, IT_MAGIC, 4))
 		return TRUE;
 
-	vfs_fseek(file, 44, SEEK_SET);
-	vfs_fread(magic, 1, 4, file);
+	if (vfs_fseek(file, 44, SEEK_SET))
+		return FALSE;
+	if (vfs_fread(magic, 1, 4, file) != sizeof magic)
+		return FALSE;
 	if (!memcmp(magic, S3M_MAGIC, 4))
 		return TRUE;
 
-	vfs_fseek(file, 1080, SEEK_SET);
-	vfs_fread(magic, 1, 4, file);   
+	if (vfs_fseek(file, 1080, SEEK_SET))
+		return FALSE;
+	if (vfs_fread(magic, 1, 4, file) != sizeof magic)
+		return FALSE;
 	if (!memcmp(magic, MOD_MAGIC_FASTTRACKER6, 4))
 		return TRUE;
 	if (!memcmp(magic, MOD_MAGIC_FASTTRACKER8, 4))
@@ -131,7 +138,7 @@ static int is_our_file_from_vfs(const gchar *filename, VFSFile *file)
 		printf("Amiga support is disabled\n");
 	}
 */
-	if (!disable_amiga_mods) {
+	if (!dumb_config.disable_amiga_mods) {
 		if (!memcmp(magic, MOD_MAGIC_PROTRACKER4, 4))
 			return TRUE;
 		if (!memcmp(magic, MOD_MAGIC_PROTRACKER4X, 4))
@@ -156,7 +163,7 @@ static int is_our_file_from_vfs(const gchar *filename, VFSFile *file)
 		if (!strcasecmp(ext, ".it")) return TRUE;
 		if (!strcasecmp(ext, ".xm")) return TRUE;
 		if (!strcasecmp(ext, ".s3m")) return TRUE;
-		if (!disable_amiga_mods) {
+		if (!dumb_config.disable_amiga_mods) {
 			if (!strcasecmp(ext, ".mod")) return TRUE;
 		}
 	}
@@ -239,22 +246,33 @@ static Tuple *get_tuple_info_from_duh(DUH *duh, const gchar *filename)
 	g_return_val_if_fail(duh != NULL, NULL);
 
 	Tuple *tuple = tuple_new_from_filename(filename);
-
-	tuple_associate_string(tuple, FIELD_FORMATTER, NULL, duh_custom_title_format);
-	tuple_associate_string(tuple, FIELD_QUALITY, NULL, "sequenced");
-	tuple_associate_int(tuple, FIELD_LENGTH, NULL, (int)(duh_get_length(duh) * 1000LL >> 16) );
+	tuple_set_str(tuple, FIELD_QUALITY, NULL, "sequenced");
+	tuple_set_int(tuple, FIELD_LENGTH, NULL, (int)(duh_get_length(duh) * 1000LL >> 16) );
 	
 	mod_title_raw = duh_get_tag(duh, "TITLE");
 	if (mod_title_raw) {
 		/* DOS (CP850) all my XM IT and S3M use this (MODs should be 7bit ASCII) */ 
 		mod_title = g_convert(mod_title_raw, -1, "UTF-8", "CP850", NULL, NULL, NULL);
-		/* Raw copy but GTK2 doesn't like none-UTF8 stuff*/ 
-		/* mod_title = g_strdup(mod_title_raw); */
-
+		/* Raw copy but GTK2 doesn't like none-UTF8 stuff
+		   mod_title = g_strdup(mod_title_raw); */
 		g_strstrip(mod_title);
 
-		if (mod_title[0])
-			tuple_associate_string(tuple, FIELD_TITLE, NULL, mod_title);
+		if (mod_title[0]) {
+			if (dumb_config.use_custom_title) {
+				gchar *custom_title, *file_title;
+				const char *basename, *basename_ext;
+
+				uri_parse(filename, &basename, &basename_ext, NULL, NULL);
+				file_title = g_strndup(basename, basename_ext - basename);
+				custom_title = g_strdup_printf("%s (%s)", file_title, mod_title);
+				tuple_set_str(tuple, FIELD_TITLE, NULL, custom_title );
+
+				g_free(custom_title);
+				g_free(file_title); 
+			} else
+				tuple_set_str(tuple, FIELD_TITLE, NULL, mod_title);
+
+		}
 
 		g_free(mod_title); mod_title = NULL;
 	}
@@ -282,7 +300,7 @@ static Tuple *duh_probe_for_tuple(const gchar *filename, VFSFile *fd)
 		return NULL; /* Return empty Tuple if module was not loadable */
 	
 	input = get_tuple_info_from_duh(duh, filename);
-	tuple_associate_string(input, FIELD_CODEC, NULL, uniread[ext_index].description);	
+	tuple_set_str(input, FIELD_CODEC, NULL, uniread[ext_index].description);	
 		
 	return input;
 }
@@ -323,7 +341,7 @@ static gboolean play_start(InputPlayback *playback, const gchar *filename,
 	{
 		install_callbacks(duh_file->file);
 		duh_file->channels = 2;
-		duh_file->samples_per_sec = output_frequency;
+		duh_file->samples_per_sec = dumb_config.output_frequency;
 		duh_file->bits_per_sample = 16;
 
 		if (playback->output->open_audio((duh_file->bits_per_sample == 16) ? FMT_S16_LE : FMT_U8, duh_file->samples_per_sec, duh_file->channels) == 0)
@@ -453,147 +471,6 @@ static void mseek(InputPlayback *playback, gint time)
 	g_mutex_unlock(control_mutex);
 }
 
-static void file_info_box(const gchar *filename)
-{
-	GtkWidget *songinfo;
-	GtkStyle *style = NULL;
-	GtkTextBuffer *TextBuffer;
-	PangoFontDescription *font_desc = NULL;
-
-	DUMB_IT_SIGDATA *sd;
-	DUH *duh;
-
-	if ( duh_universal_load_vfs(&duh, filename, NULL, UNIREAD_MOD) == -1 )
-		return;
-	
-	sd = duh_get_it_sigdata(duh);
-
-	songinfo = create_songinfo ();
-	if (!songinfo) return;
-
-	font_desc = pango_font_description_from_string ("monospace 9");
-
-	if (font_desc) {
-		style = gtk_style_copy(gtk_widget_get_style(songinfo));
-		style->font_desc = font_desc;
-	}
-	
-	{
-		int n = dumb_it_sd_get_n_instruments(sd);
-		if (n) {
-			GtkCList *l = GTK_CLIST(lookup_widget(songinfo, "songinfo_instruments_clist"));
-			int i;
-			if (style) gtk_widget_set_style(GTK_WIDGET(l), style);
-			gtk_clist_column_titles_passive(l);
-			gtk_clist_set_column_justification(l, 0, GTK_JUSTIFY_RIGHT);
-			for (i = 0; i < n; i++) {
-				char str[3];
-				const gchar *text[] = {
-					str,
-					(gchar*) dumb_it_sd_get_instrument_name(sd, i),
-					(gchar*) dumb_it_sd_get_instrument_filename(sd, i)
-				};
-				sprintf(str, "%d", i+1);
-				gtk_clist_insert(l, i, (gchar **)text);
-			}
-		} else
-			gtk_notebook_remove_page(GTK_NOTEBOOK(lookup_widget(songinfo, "songinfo_notebook")), 2);
-	}
-
-	{
-		GtkCList *l = GTK_CLIST(lookup_widget(songinfo, "songinfo_samples_clist"));
-		int i, n = dumb_it_sd_get_n_samples(sd);
-		if (style) gtk_widget_set_style(GTK_WIDGET(l), style);
-		gtk_clist_column_titles_passive(l);
-		gtk_clist_set_column_justification(l, 0, GTK_JUSTIFY_RIGHT);
-		for (i = 0; i < n; i++) {
-			char str[3];
-			const gchar *text[] = {
-				str,
-				(gchar*) dumb_it_sd_get_sample_name(sd, i),
-				(gchar*) dumb_it_sd_get_sample_filename(sd, i)
-			};
-			sprintf(str, "%d", i+1);
-			gtk_clist_insert(l, i, (gchar **)text);
-		}
-	}
-
-	{
-		GtkTextView *l = GTK_TEXT_VIEW(lookup_widget(songinfo, "songinfo_message_text"));
-		const char *message = (char*) dumb_it_sd_get_song_message(sd);
-		if (message) {
-			char *m = strdup(message);
-			int i;
-			for (i = 0; m[i]; i++)
-				if (m[i] == '\r') m[i] = '\n';
-
-			TextBuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (lookup_widget(songinfo, "songinfo_message_text")));
-			if (style) gtk_widget_set_style(GTK_WIDGET(l), style);
-			gtk_text_buffer_set_text(TextBuffer, (gchar*) m, -1);
-
-			free(m);
-		} else
-			gtk_notebook_remove_page(GTK_NOTEBOOK(lookup_widget(songinfo, "songinfo_notebook")), 0);
-	}
-
-	/* I hope that's ok here. Previously there was no unload_duh at all (leak?) */
-	if (duh)
-		unload_duh(duh);
-
-	gtk_widget_show (songinfo);
-}
-
-static void duh_configure(void) {
-	if (prefs) return;
-	prefs = create_prefs();
-	if (!prefs) return;
-
-	gint output_frequency_idx = 4; /* 44100 default/fallback */
-	int i;
-	gchar freqstring[6];
-
-	for (i = 0; i < (int) (sizeof(output_frequencies) / sizeof(output_frequencies[0])); i++) {
-		snprintf(freqstring, sizeof(freqstring), "%d", output_frequencies[i]);
-		gtk_combo_box_append_text (GTK_COMBO_BOX(lookup_widget(prefs, "prefs_freq_combobox")), freqstring);
-		if (output_frequencies[i] == output_frequency)
-			output_frequency_idx = i;
-	}
-	
- 	gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget(prefs, "prefs_freq_combobox")), output_frequency_idx);
-
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(prefs, "prefs_checkbox_disableamiga")), disable_amiga_mods);
-
-	gtk_signal_connect(GTK_OBJECT(prefs), "destroy",         
-			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &prefs);
-
-	gtk_signal_connect_object(GTK_OBJECT(lookup_widget(prefs, "prefs_button_cancel")), "clicked",
-				  GTK_SIGNAL_FUNC(gtk_widget_destroy), 
-				  GTK_OBJECT(prefs));
-
-	gtk_signal_connect_object(GTK_OBJECT(lookup_widget(prefs, "prefs_button_ok")), "clicked",
-				  GTK_SIGNAL_FUNC(duh_configure_ok_cb), 
-				  NULL);
-
-	gtk_widget_show(prefs);
-}
-
-static void duh_configure_ok_cb(GtkButton *w, gpointer data) {
-	mcs_handle_t *db;
-	gint output_frequency_idx;
-
-	db = aud_cfg_db_open();
-
-	disable_amiga_mods = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(prefs, "prefs_checkbox_disableamiga")));
-	aud_cfg_db_set_bool(db, "DUMB", "disable_amiga_mods", disable_amiga_mods);
-
-	output_frequency_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget(prefs, "prefs_freq_combobox")));
-	output_frequency = output_frequencies[output_frequency_idx];
-	aud_cfg_db_set_int(db, "DUMB", "output_frequency", output_frequency);	
-
-	aud_cfg_db_close(db);
-	gtk_widget_destroy(prefs);
-}
-
 static void duh_about(void) {
 	static GtkWidget *about;
 	gchar *about_text;
@@ -614,3 +491,277 @@ static void duh_about(void) {
 
 	g_free(about_text);
 }
+
+/* Preferences Window */ 
+
+static void
+configure_init(void)
+{
+	memcpy(&dumb_config_gui, &dumb_config, sizeof(dumb_config_gui));
+}
+
+static void
+configure_cleanup(void)
+{
+}
+
+static void
+configure_apply()
+{
+	memcpy(&dumb_config, &dumb_config_gui, sizeof(dumb_config));
+
+	aud_set_bool("DUMB", "disable_amiga_mods", dumb_config.disable_amiga_mods);
+	aud_set_int("DUMB", "output_frequency", dumb_config.output_frequency);
+	aud_set_bool("DUMB", "use_custom_title", dumb_config.use_custom_title);
+}
+
+static ComboBoxElements frequency_elements[] = {
+	{ GINT_TO_POINTER(22050), "22050" },
+	{ GINT_TO_POINTER(44100), "44100" },
+	{ GINT_TO_POINTER(48000), "48000" },
+	{ GINT_TO_POINTER(64000), "64000" },
+	{ GINT_TO_POINTER(96000), "96000" },
+	{ GINT_TO_POINTER(192000), "192000" },
+};
+
+static PreferencesWidget settings[] = {
+	{WIDGET_CHK_BTN, N_("Disable Amiga Support"),
+		.cfg_type = VALUE_BOOLEAN,
+		.cfg = &dumb_config_gui.disable_amiga_mods,
+		.tooltip = N_("Use this if you have a better decoder for 4 "
+		              "channel Amiga MOD files (like UADE)"),
+	},
+	{WIDGET_COMBO_BOX, N_("Frequency:"),
+		.cfg_type = VALUE_INT,
+		.cfg = &dumb_config_gui.output_frequency,
+		.data = {.combo = {frequency_elements, G_N_ELEMENTS (frequency_elements), TRUE}},
+		.tooltip = N_("If you use dmix, pulseaudio or a similar stream "
+		              "mixer, choose the same frequency here to avoid "
+			      "useless resampling, otherwise pick whatever quality "
+			      "you like"),
+	},
+	{WIDGET_SEPARATOR, .data = {.separator = {TRUE}}},
+	{WIDGET_CHK_BTN, N_("Use \"Filename (Real Mod Title)\" as title"),
+		.cfg_type = VALUE_BOOLEAN,
+		.cfg = &dumb_config_gui.use_custom_title,
+		.tooltip = N_("Recommended because many MOD titles are "
+		              "stylized, contain trailing or leading spaces or even "
+		              "ASCII art which is very bad for playlist sorting"),
+	},
+	{WIDGET_SEPARATOR, .data = {.separator = {TRUE}}},
+};
+
+static PluginPreferences preferences = {
+	.domain = PACKAGE,
+	.title = N_("DUMB Preferences"),
+	.prefs = settings,
+	.n_prefs = G_N_ELEMENTS(settings),
+	.type = PREFERENCES_WINDOW,
+	.init = configure_init,
+	.apply = configure_apply,
+	.cleanup = configure_cleanup,
+};
+
+static void close_window(GtkWidget *widget, gpointer win) {
+	gtk_widget_destroy(GTK_WIDGET(win));
+}
+
+static gboolean escape_close(GtkWidget *widget, GdkEventKey *event, gpointer data) {
+	if (event->keyval == GDK_Escape) {
+		gtk_widget_destroy(GTK_WIDGET(widget));
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void file_info_box(const gchar *filename) {
+	GtkWidget *window;
+
+	GtkWidget *window_vbox;
+	GtkWidget *notebook;
+
+	GtkWidget *label1, *label2, *label3;
+	GtkWidget *child1, *child2, *child3;
+	GtkWidget *child1_scroll, *child2_scroll, *child3_scroll;
+
+	GtkWidget *close_hbox, *close_button;
+
+	GtkTextIter child1_iter;
+	GtkTextBuffer *child1_buffer;
+
+	GtkTreeIter child2_iter, child3_iter;
+	GtkListStore *child2_store, *child3_store;
+
+	PangoFontDescription *font_desc;
+
+	const gchar *message, *basename;
+	gchar *msg_clean, *window_title;
+	int i, n, ext_index;
+
+	DUMB_IT_SIGDATA *sd;
+	DUH *duh;
+
+	ext_index = duh_universal_load_vfs(&duh, filename, NULL, UNIREAD_MOD);
+	if (ext_index == -1)
+		return;
+
+	sd = duh_get_it_sigdata(duh);
+
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+
+	uri_parse(filename, &basename, NULL, NULL, NULL);
+	window_title = g_strdup_printf("Track Information - %s (%s)", basename, uniread[ext_index].description);
+	gtk_window_set_title (GTK_WINDOW (window), window_title);
+	g_free(window_title);
+
+	gtk_container_set_border_width (GTK_CONTAINER (window), 10);
+	gtk_widget_set_size_request (window, 640, 480);
+	g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(close_window), window);
+	g_signal_connect(G_OBJECT(window), "key-press-event", G_CALLBACK(escape_close), NULL);
+	window_vbox = gtk_vbox_new(FALSE, 6);
+	gtk_container_add (GTK_CONTAINER (window), window_vbox);
+
+	font_desc = pango_font_description_from_string ("monospace");
+
+	/* ========= content child widgets for the 3 tabs =========*/
+
+	notebook = gtk_notebook_new();
+	gtk_box_pack_start(GTK_BOX(window_vbox), notebook, TRUE, TRUE, 4);
+
+	/* --- tab 1 content (text view) --- */
+	message = (gchar*) dumb_it_sd_get_song_message(sd);
+	if (message) {
+		child1_scroll = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(child1_scroll), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS );
+		child1 = gtk_text_view_new();
+		gtk_text_view_set_editable(GTK_TEXT_VIEW(child1), FALSE);
+		gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(child1), FALSE);
+		gtk_container_add(GTK_CONTAINER(child1_scroll), child1);
+
+		child1_buffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(child1) );
+		gtk_text_buffer_get_iter_at_offset(child1_buffer, &child1_iter, 0);
+
+		gtk_text_buffer_create_tag(child1_buffer, "lmarg", "left_margin", 5, NULL);
+
+		/* add message textview content */
+
+		/* msg_clean = g_strdup(message); */
+		msg_clean = g_convert(message, -1, "UTF-8", "CP850", NULL, NULL, NULL);
+		for (i = 0; msg_clean[i]; i++)
+			if (msg_clean[i] == '\r') msg_clean[i] = '\n';
+
+		gtk_text_buffer_insert_with_tags_by_name(child1_buffer, &child1_iter,
+				msg_clean, -1, "lmarg",  NULL);
+
+		g_free(msg_clean);
+
+		gtk_widget_modify_font(child1, font_desc);
+
+		label1 = gtk_label_new("Message");
+		gtk_notebook_append_page(GTK_NOTEBOOK (notebook), child1_scroll, label1);
+	}
+
+	/* --- tab 2 content (tree view) --- */
+
+	n = dumb_it_sd_get_n_samples(sd);
+
+	if (n) {
+		child2_scroll = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(child2_scroll), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS );
+		child2 = gtk_tree_view_new();
+		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(child2), TRUE);
+		gtk_container_add(GTK_CONTAINER(child2_scroll), child2);
+
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(child2), -1,
+				"#",
+				gtk_cell_renderer_text_new(),
+				"text", COL_N, NULL);
+
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(child2), -1,
+				"Sample Name",
+				gtk_cell_renderer_text_new(),
+				"text", COL_NAME, NULL);
+
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(child2), -1,
+				"File Name",
+				gtk_cell_renderer_text_new(),
+				"text", COL_FILE, NULL);
+
+		child2_store = gtk_list_store_new(NUM_COLS, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING);
+
+		for (i = 0; i < n; i++) {
+			gtk_list_store_append(child2_store, &child2_iter);
+			gtk_list_store_set(child2_store, &child2_iter,
+					COL_N, i+1,
+					COL_NAME, (gchar*) dumb_it_sd_get_sample_name(sd, i),
+					COL_FILE, (gchar*) dumb_it_sd_get_sample_filename(sd, i),
+					-1);
+		}
+		gtk_tree_view_set_model(GTK_TREE_VIEW(child2), GTK_TREE_MODEL(child2_store));
+
+		gtk_widget_modify_font(child2, font_desc);
+
+		label2 = gtk_label_new("Samples");
+		gtk_notebook_append_page(GTK_NOTEBOOK (notebook), child2_scroll, label2);
+	}
+	/* --- tab 3 content (tree view) --- */
+
+	n = dumb_it_sd_get_n_instruments(sd);
+
+	if (n) {
+		child3_scroll = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(child3_scroll), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS );
+		child3 = gtk_tree_view_new();
+		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(child3), TRUE);
+		gtk_container_add(GTK_CONTAINER(child3_scroll), child3);
+
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(child3), -1,
+				"#",
+				gtk_cell_renderer_text_new(),
+				"text", COL_N, NULL);
+
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(child3), -1,
+				"Instrument Name",
+				gtk_cell_renderer_text_new(),
+				"text", COL_NAME, NULL);
+
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(child3), -1,
+				"File Name",
+				gtk_cell_renderer_text_new(),
+				"text", COL_FILE, NULL);
+
+		child3_store = gtk_list_store_new(NUM_COLS, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING);
+
+		for (i = 0; i < n; i++) {
+			gtk_list_store_append(child3_store, &child3_iter);
+			gtk_list_store_set(child3_store, &child3_iter,
+					COL_N, i+1,
+					COL_NAME, (gchar*) dumb_it_sd_get_instrument_name(sd, i),
+					COL_FILE, (gchar*) dumb_it_sd_get_instrument_filename(sd, i),
+					-1);
+		}
+
+		gtk_tree_view_set_model(GTK_TREE_VIEW(child3), GTK_TREE_MODEL(child3_store));
+
+		gtk_widget_modify_font(child3, font_desc);
+
+		label3 = gtk_label_new("Instrument");
+		gtk_notebook_append_page(GTK_NOTEBOOK (notebook), child3_scroll, label3);
+	}
+
+	/* close button within hbox */
+
+	close_hbox = gtk_hbox_new(FALSE, 6);
+	close_button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	gtk_box_pack_end(GTK_BOX(close_hbox), close_button, FALSE, FALSE, 6);
+	g_signal_connect(G_OBJECT(close_button), "clicked", G_CALLBACK(close_window), window);
+
+	gtk_box_pack_start(GTK_BOX(window_vbox), close_hbox, FALSE, FALSE, 2);
+
+	/* */
+
+	if (duh)
+		unload_duh(duh);
+
+	gtk_widget_show_all(window);
+}
